/*
 * Call.java
 *
 * Created on February 6, 2003, 10:06 AM
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU GPL (General Public License) as published
 * by the Free Software Fondation:
 * http://www.gnu.org/copyleft/gpl.html
 */

package com.hbaastrup.hermod;
import java.net.*;
import java.io.*;
import java.util.*;


/**
 *
 * @author  Henrik Baastrup (c) Feb 2003
 */
public class Call
{
    private Vector params = null;
    private String methodName = null;
    private String urlString = null;
    private String connectionId = null;
    private java.io.BufferedOutputStream out = null;
    private java.io.BufferedReader in = null;
    
    public Call()
    {
    }//Call
    
    
    public void setTagetUrl(String aUrl)
    {
        urlString = aUrl;
    }//setTagetUri
    
    
    public void setMethodName(String aName)
    {
        methodName = aName;
    }//setMethod
    
    
    public void setParams(Vector params)
    {
        this.params = new Vector(params);
    }//setProperty
    
    
    public void setConnectionId(String aId)
    {
        connectionId = aId;
    }//setConnectionId
    
    
    public String getConnectionId()
    {
        return connectionId;
    }//getConnectionId
    
    
    public com.hbaastrup.hermod.Response setUserPassword(String user, String password) throws com.hbaastrup.hermod.HermodException
    {
        Vector oldParams = params;
        String oldMethodName = methodName;
        com.hbaastrup.hermod.Response response;
        com.hbaastrup.hermod.Parameter param;
        
        methodName = "set-user-password";
        params = new Vector(0);
        param = new com.hbaastrup.hermod.Parameter("user", user);
        params.addElement(param);
        param = new com.hbaastrup.hermod.Parameter("password", password);
        params.addElement(param);
        
        response = invoke();
        
        params = oldParams;
        methodName = oldMethodName;
        
        return response;
    }//setUserPassword
    
    
    public com.hbaastrup.hermod.Response getFreeConnection() throws com.hbaastrup.hermod.HermodException
    {
        Vector oldParams = params;
        String oldMethodName = methodName;
        com.hbaastrup.hermod.Response response;

        methodName = "get-free-connection";
        params = null;
        
        response = invoke();
        
        params = oldParams;
        methodName = oldMethodName;
        
        return response;
    }//getFreeConnection

    
    public com.hbaastrup.hermod.Response getConnection() throws com.hbaastrup.hermod.HermodException
    {
        Vector oldParams = params;
        String oldMethodName = methodName;
        com.hbaastrup.hermod.Response response;

        methodName = "get-connection";
        params = null;
        
        response = invoke();
        
        params = oldParams;
        methodName = oldMethodName;
        
        return response;
    }//getConnection

    
    public com.hbaastrup.hermod.Response releaseConnection() throws com.hbaastrup.hermod.HermodException
    {
        Vector oldParams = params;
        String oldMethodName = methodName;
        com.hbaastrup.hermod.Response response;

        methodName = "release-connection";
        params = null;
        
        response = invoke();
        connectionId = null;
        
        params = oldParams;
        methodName = oldMethodName;
        
        return response;
    }//releaseConnection

    
    public com.hbaastrup.hermod.Response invoke() throws com.hbaastrup.hermod.HermodException
    {
        StringBuffer xmlRpcStr = new StringBuffer("<?xml version=\"1.0\"?>\n");
        StringBuffer responseStr;
        String line, attr;
        com.hbaastrup.hermod.Parameter param;
        Enumeration enum;
        URL url;
        URLConnection urlConn;
        com.hbaastrup.hermod.Response response = null;
        com.hbaastrup.MiniXML.MiniXML miniXml = new com.hbaastrup.MiniXML.MiniXML();
        com.hbaastrup.MiniXML.Element xmlTree, elm, subElm;
        Vector vect, responseParams;
        int i, j;
        
        // Build the XML RPC string
        if (connectionId==null) xmlRpcStr.append("<methodCall>\n");
        else xmlRpcStr.append("<methodCall connection-id='"+connectionId+"'>\n");
        xmlRpcStr.append(" <methodName>"+methodName+"</methodName>\n");
        xmlRpcStr.append(" <params>\n");
        
        if (params!=null) {
            enum = params.elements();
            while (enum.hasMoreElements()) {
                param = (com.hbaastrup.hermod.Parameter)enum.nextElement();
                xmlRpcStr.append("  <param name='"+param.name+"'>\n");
                xmlRpcStr.append("   <string>"+param.value+"</string>\n");
                xmlRpcStr.append("  </param>\n");
            }
        }
        
        xmlRpcStr.append(" </params>\n");
        xmlRpcStr.append("</methodCall>\n");
        
        try {
            // Open the connection
            url = new URL(urlString);
            urlConn = url.openConnection();
            urlConn.setDoInput(true);
            urlConn.setDoOutput(true);
            urlConn.setUseCaches(false);
            
            // Send the XML RPC string
            out = new BufferedOutputStream(urlConn.getOutputStream());
            out.write(xmlRpcStr.toString().getBytes());
            out.flush();
            
            // Let get the response
            responseStr = new StringBuffer();
            in = new BufferedReader(new InputStreamReader(urlConn.getInputStream()));
            while ((line=in.readLine())!=null) {
                responseStr.append(line);
//                System.out.println(line);
            }
            
            // Close down the line
            out.close();
            out = null;
            in.close();
            in = null;
        } catch (Exception ex) {
            HermodException hEx = new HermodException(ex.toString());
            throw hEx;
        }
        
        // Interpt the XML response string
        response = new com.hbaastrup.hermod.Response();
        miniXml.parse(responseStr.toString());
        xmlTree = miniXml.getTree();
//        miniXml.writeOutTree(System.out, xmlTree);
        xmlTree.initSearch();
        if ((elm=xmlTree.searchFor("fault"))!=null) { //the server responded with a fault
            com.hbaastrup.hermod.Fault fault = new com.hbaastrup.hermod.Fault();
            
            if ((elm=xmlTree.searchFor("struct"))==null) { //<struct>
                HermodException ex = new HermodException("The server responded with a fault. But there are syntax errors in it.");
                throw ex;
            }
            vect = elm.getSubElements(); // 
            for (i=0; i<2; i++) { //we have only to values in a fault
                elm = (com.hbaastrup.MiniXML.Element)vect.get(i); //<member>
                if ((subElm=elm.getFirstSubElement())!=null) { //<name>
                    if (subElm.getValue().equals("faultCode")) {
                        if ((subElm=elm.getNextSubElement())!=null) { //<value>
                            if ((subElm=subElm.getFirstSubElement())!=null) { //<int>
                                fault.setFaultCode(new Integer(subElm.getValue()).intValue());
                                continue;
                            }
                        }
                    }
                    else if (subElm.getValue().equals("faultString")) {
                        if ((subElm=elm.getNextSubElement())!=null) { //<value>
                            if ((subElm=subElm.getFirstSubElement())!=null) { //<string>
                                fault.setFaultString(subElm.getValue());
                                continue;
                            }
                        }
                    }
                }
                HermodException ex = new HermodException("The server responded with a fault. But there are syntax errors in it.");
                throw ex;
            }
            response.setFault(fault);
            return response;
        }
        
        if ((elm=xmlTree.searchFor("methodResponse"))==null) { //<methodResponse>
            HermodException ex = new HermodException("There is a syntax error in the server response.");
            throw ex;
        }
        if ((line=elm.getAttribute("connection-id"))!=null) {
            connectionId = line;
            response.setConnectionId(line);
        }
        else connectionId = null;
        
        if ((elm=xmlTree.searchFor("params"))==null) { //<params>
            HermodException ex = new HermodException("There is a syntax error in the server response.");
            throw ex;
        }
        if ((vect=elm.getSubElements())!=null) {
            enum = vect.elements();
            responseParams = new Vector(0);
            while (enum.hasMoreElements()) {
                elm = (com.hbaastrup.MiniXML.Element)enum.nextElement(); //<parm>
                if ((attr=elm.getAttribute("name"))!=null) {
                    param = new com.hbaastrup.hermod.Parameter();
                    param.name = attr;
                    if ((subElm = elm.getFirstSubElement())!=null) { //<value>
                        if ((subElm = subElm.getFirstSubElement())!=null) { //<string>
                            param.value = subElm.getValue();
                            responseParams.add(param);
                            continue;
                        }
                    }
                }
                HermodException ex = new HermodException("There is a syntax error in the server response.");
                throw ex;
            }
            response.setParams(responseParams);
        }
        else response.setParams(null);
            
        return response;
    }//invoke
    
    public void close() throws com.hbaastrup.hermod.HermodException
    {
        try {
            if (out!=null) out.close();
            out = null;
            if (in!=null) in.close();
            in = null;
        } catch (Exception ex) {
            HermodException hEx = new HermodException(ex.toString());
            throw hEx;
        }
    }//close

}//Call
