/***************************************************************************
                          LinuxMain.cpp  -  description
                             -------------------
    begin                : Fri Jan 3 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef LINUX

#include <signal.h>
#include <stdlib.h>
#include "Version.h"
#include "Hermod.h"
#include "ScreenConnection/Helpers.h"

extern int HermodBind(char *, char *, char *, short, char *);
extern void StopHermod(short);


char HermodConfigFile[_MAX_PATH];
char XmlConfigFileName[_MAX_PATH];
char HttpLogFileName[_MAX_PATH];
char PidFileName[_MAX_PATH];
char MainframeUser[15];
char MainframePassword[15];
short HttpPort;
int UsePassedHttpPort = 0;



void PrintHelp()
{
	printf("\n");
	printf("Hermod syntax:\n");
	printf("\\nHermod [-g|-f file|-p port|-v|-h]\n");
	printf("\nExplanation:\n");
	printf(" -g: run Hermod interactive and not as a demon\n");
	printf(" -f: file: use the file as configoration file.\n");
	printf(" -p: port: use the port as HTTP listen port\n");
	printf(" -v: print the version of the application.\n");
	printf(" -s: stop the server daemon\n");
	printf(" -h: print this help text.\n");
}//PrintHelp


int ReadConfigFile(char *filename)
{
	FILE *fd;
	char str[256], *cp;

	if ((fd=fopen(filename, "r"))!=NULL) {
		while (fgets(str, 255, fd)) {
			if (*str=='#') continue; // Comment
			if (strncmp(str,"XmlConfigFile:",14)==0) {
				cp = strtrim(&str[14]);
				strcpy(XmlConfigFileName, cp);
			}
			else if (strncmp(str,"HttpLogFile:",12)==0) {
				cp = strtrim(&str[12]);
				strcpy(HttpLogFileName, cp);
			}
			else if (strncmp(str,"PidFile:",8)==0) {
				cp = strtrim(&str[8]);
				strcpy(PidFileName, cp);
			}
			else if (!UsePassedHttpPort && strncmp(str,"HttpPort:",9)==0) {
				cp = strtrim(&str[9]);
				HttpPort = (short)atol(cp);
			}
			else if (strncmp(str,"MainframUser:",13)==0) {
				cp = strtrim(&str[13]);
				strcpy(MainframeUser, cp);
			}
			else if (strncmp(str,"MainframePassword:",18)==0) {
				cp = strtrim(&str[18]);
				strcpy(MainframePassword, cp);
			}
		}
		fclose(fd);
		return 0;
	}
	return -1;
}//ReadConfigFile


void SigHupHandler(int arg)
// Handler for SIGHUP (hangup) signal
{
	printf("We got a SIGHUP signal!\n");
//	ReadConfigFile(HermodConfigFile); // re-read the config file
}//SigHupHandler


void SigTermHandler(int arg)
// Handler for SIGTERM (sw termination) signal
{
	sigset_t sigSet;
	
	sigemptyset(&sigSet);
	sigaddset(&sigSet, SIGTERM); //Allow the SW terminate signal!
	sigprocmask(SIG_UNBLOCK, &sigSet, NULL);
	
	StopHermod(HttpPort);
	exit(EXIT_SUCCESS);
}//SigTermHandler


int main(int argc, char *argv[])
{
	FILE *fd;
	pid_t pid, sid;
	int runInteractive = 0;
	int stopDaemon = 0;
	int reTryReadConfigFile = 1;
	sigset_t sigSet;
	struct sigaction termAction;
	char *cp, str[256];


	strcpy(HermodConfigFile, "/etc/hermod.conf");
	strcpy(XmlConfigFileName, "/etc/config.xml");
	strcpy(HttpLogFileName, "/var/log/HermodHttp.log");
	strcpy(PidFileName, "/tmp/hermod.pid");
	HttpPort = 5080;
	strcpy(MainframeUser,"USER");
	strcpy(MainframePassword,"PASSWORD");

	// Interpret the program aruments
	for (int i=1; i<argc; i++) {
		if (strcmp(argv[i],"-h")==0) {
			PrintHelp();
			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i],"-v")==0) {
			printf("\n%s\n",VERSION);
#if defined (USE_PERCOMM)
			printf("  for IBM's Personal Communications Emulator\n");
#elif defined(USE_X3270)
			printf("  for X-3270 Emulator\n");
#elif defined(USE_EXEMUL)
			printf("  for External Windows Emulator\n");			
#endif
			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i],"-f")==0) {
			i++;
			if (i < argc) {
				reTryReadConfigFile = 0;
				strcpy(HermodConfigFile, argv[i]);
			}
		}
		else if (strcmp(argv[i],"-p")==0) {
			i++;
			if (i < argc) {
				HttpPort = (short)atol(argv[i]);
				UsePassedHttpPort = 1;
			}
		}
		else if (strcmp(argv[i],"-g")==0) {
			runInteractive = 1;
		}
		else if (strcmp(argv[i],"-s")==0) {
			stopDaemon = 1;
		}
	}

	// Read configuration
	if (ReadConfigFile(HermodConfigFile)!=EXIT_SUCCESS) {
		if (reTryReadConfigFile) {
			strcpy(HermodConfigFile, "/etc/opt/hermod/hermod.conf");
			if (ReadConfigFile(HermodConfigFile)!=EXIT_SUCCESS) exit(EXIT_FAILURE);
		}
		else exit(EXIT_FAILURE);
	}

	// Stop the deamon!
	if (stopDaemon) {
		if ((fd=fopen(PidFileName,"r"))==NULL) exit(EXIT_FAILURE);
		fgets(str, 80, fd);
		fclose(fd);
		cp = strtrim(str);
		pid = atol(cp);
		kill(pid, SIGTERM); // first we tryu gratfull
		sleep(60);
		kill(pid, SIGKILL); // we just want to be sure it is dead!
		return EXIT_SUCCESS;
	}

	if (runInteractive) {
		printf("Hermod starting\n%s\n",VERSION);
//		HermodBind("/home/hbaastrup/projects/hermod/config.xml", "CS4AA06", "abcde01", 5080,"/tmp/HermodHttp.log");
		HermodBind(XmlConfigFileName, MainframeUser, MainframePassword, HttpPort, HttpLogFileName);
		return EXIT_SUCCESS;
	}

	// Daemon stuff
	pid = fork();
	if (pid < 0) exit(EXIT_FAILURE);
	if (pid > 0) {
		exit(EXIT_SUCCESS); //The parrent exit
	}

	if ((sid=setsid()) < 0) exit(EXIT_FAILURE); //Start a new session
	if (chdir("/") < 0) exit(EXIT_FAILURE); //Make  / the current directory
	umask(0); //Reset the file mode

	//Close unneeded file descriptors
	close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);
	
	// write PID to file
	if ((fd=fopen(PidFileName,"w"))==NULL) exit(EXIT_FAILURE);
	fprintf(fd,"%d\n",getpid());
	fclose(fd);

	// Some signal we want to handle
	sigemptyset(&sigSet);
	sigaddset(&sigSet, SIGTERM); //Allow the SW terminate signal!
	sigprocmask(SIG_UNBLOCK, &sigSet, NULL);
	sigemptyset(&sigSet);
	termAction.sa_handler = SigTermHandler;
	termAction.sa_mask = sigSet;
	termAction.sa_flags = SA_ONESHOT;
	if (sigaction(SIGTERM, &termAction, NULL) < 0) exit(EXIT_FAILURE); //Set action for SW terminate signal

	//Start our work
	HermodBind(XmlConfigFileName, MainframeUser, MainframePassword, HttpPort, HttpLogFileName);

	return EXIT_SUCCESS;
}//main



#endif
