/***************************************************************************
                          MiniXML.cpp  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/

#include "MiniXML.h"

void CleanStr(char *str)
{
	char *cp1, *cp2;

	cp1 = cp2 = str;
	while (*cp2) {
		if (*cp2==' '||*cp2=='"'||*cp2=='\'') cp2++;
		else {
			*cp1 = *cp2;
			cp1++;
			cp2++;
		}
	}
	*cp1 = '\0';
}//CleanStr


MiniXML::MiniXML()
/**
 * Creator
 */
{
	mXmlTree = new Element();;
	mOutStr = NULL;
	mOutStrBlocks = 0;
	mFirstCall = 0;
}//MiniXML

MiniXML::~MiniXML()
/**
 * Destructor
 */
{
	DropTree();
	delete mXmlTree;
	if (mOutStr) free(mOutStr);
	mOutStr = NULL;
}//~MiniXML

void MiniXML::DropTree()
/**
 * Clean-up the whole XML docoment tree. And make the class ready for a new parse call.
 */
{
//	Element *actElm, *clearElm;

	if (mXmlTree==NULL) return;
	mXmlTree->DeleteSubElements();
/*	actElm = mXmlTree->GetFirstSubElement();
	while (actElm) {
		actElm->DeleteSubElements();
		clearElm = actElm;
		actElm = actElm->GetNextElement();
		delete clearElm;
	}*/
	delete mXmlTree;
	mXmlTree = new Element();
}//DropTree

char *MiniXML::CleanUpString(char *str)
/**
 * Clean-up the input string for not warnded charaters.
 */
{
	char *retStr, *cp, *rcp;

	rcp = retStr = (char *)malloc(strlen(str)+1);
	if (rcp==NULL) return NULL;
	cp = str;
	while (*cp) {
		switch (*cp) {
			case '\r':
			case '\n':
				break;
			default:
				*rcp = *cp;
				rcp++;
		}
		cp++;
	}
	*rcp = '\0';
	return retStr;
}//CleanUpString

char *MiniXML::Parse(char *xmlStr, Element *elm)
/**
 * Parse state engine. It is here the actull parsing is done
 */
{
	int stop, state, noStep, pingState, oldState, ignore;
	char *tagName = NULL;
	char *tagValue = NULL;
	char *attrName = NULL;
	char *attrValue = NULL;
	char *endTagName = NULL;
	Element *subElm = NULL;
	ElementAttribute *attr = new ElementAttribute();
	char *cpStr, *cpTag, *pStr;

	stop = 0;
	noStep = 0;
	state = NEW_TAG_STATE;
	pingState = 0;
	ignore = 0;
	cpStr = (char *)xmlStr;
	while (*cpStr && !stop) {
		switch (state) {
			case NEW_TAG_STATE: //New tag!
				if (ignore) { //If we are in an ignore state change state for the next tag.
					if (*cpStr=='>') ignore = 0;
				}
				else if (*cpStr=='<') {
					if (*(cpStr+1)=='/') {
						if (endTagName) free(endTagName);
						if ((endTagName=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*endTagName = '\0';
						cpTag = endTagName;
						state = GET_END_TAG_NAME_STATE;
						cpStr++;
					}
					else if (*(cpStr+1)=='?') {
						oldState = state;
						state = TAG_DECLARATION_STATE;
					}
					else if (*(cpStr+1)=='!') {
						if (*(cpStr+2)=='-' && *(cpStr+3)=='-') {
							oldState = state;
							state = TAG_COMMENT_STATE;
						}
						else ignore = 1;
					}
					else {
						if (tagName) free(tagName);
						if ((tagName=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*tagName = '\0';
						cpTag = tagName;
						state = GET_TAG_NAME_STATE;
					}
				}
				break;

			case GET_TAG_NAME_STATE: //Get begin tag name
				if (*cpStr==' ' || *cpStr=='\t') {
					*cpTag = '\0';
					elm->SetName(tagName);
					if (attrName) free(attrName);
					if ((attrName=(char *)malloc(strlen(cpStr)))==NULL) continue;
					*attrName = '\0';
					cpTag = attrName;
					state = GET_ATTR_NAME_STATE;
					break;
				}
				if (*cpStr=='>') {
					*cpTag = '\0';
					elm->SetName(tagName);
					if (tagValue) free(tagValue);
					if ((tagValue=(char *)malloc(strlen(cpStr)))==NULL) continue;
					*tagValue = '\0';
					cpTag = tagValue;
					state = GET_TAG_VALUE_STATE;
					break;
				}
				if (*cpStr=='/' && *(cpStr+1)=='>') {
					*cpTag = '\0';
					elm->SetName(tagName);
					cpStr++;
					state = TAG_END_STATE;
					break;
				}
				*cpTag = *cpStr;
				cpTag++;
				break;

			case GET_ATTR_NAME_STATE: //Get attribute name
				if (*cpStr==' ' || *cpStr=='\t') break;
				if (*cpStr=='=') {
					*cpTag = '\0';
					if (attr->mName) free(attr->mName);
					if ((attr->mName=(char *)malloc(strlen(attrName)+1))==NULL) continue;
					strcpy(attr->mName,attrName);
					if (attrValue) free(attrValue);
					if ((attrValue=(char *)malloc(strlen(cpStr)))==NULL) continue;
					*attrValue = '\0';
					cpTag = attrValue;
					state = GET_ATTR_VALUE_STATE;
					break;
				}
				if (*cpStr=='>') {
					*cpTag = '\0';
					if (attr->mName) free(attr->mName);
					if ((attr->mName=(char *)malloc(strlen(attrName)+1))==NULL) continue;
					strcpy(attr->mName,attrName);
					if (tagValue) free(tagValue);
					if ((tagValue=(char *)malloc(strlen(cpStr)))==NULL) continue;
					*tagValue = '\0';
					cpTag = tagValue;
					state = GET_TAG_VALUE_STATE;
					break;
				}
				if (*cpStr=='/' && *(cpStr+1)=='>') {
					*cpTag = '\0';
					if (attr->mName) free(attr->mName);
					if ((attr->mName=(char *)malloc(strlen(attrName)+1))==NULL) continue;
					strcpy(attr->mName,attrName);
					elm->AddAttribute(attr);
					attr = new ElementAttribute();
					cpStr++;
					state = TAG_END_STATE;
					break;
				}
				*cpTag = *cpStr;
				cpTag++;
				break;

			case GET_ATTR_VALUE_STATE: //Get attribute value
				if (*cpStr=='\'' || *cpStr=='\"') {
					int cont = 0;
					switch (pingState) { //ping = '
						case 0:
							if (*cpStr=='\'') pingState = 1; // we have a ' and expect the attribute end with a '
							else if (*cpStr=='\"') pingState = 2; // we have a " and expect the attribute end with a "
							break;
						case 1:
							if (*cpStr=='\'') pingState = 0;
							else cont = 1;
							break;
						case 2:
							if (*cpStr=='\"') pingState = 0;
							else cont = 1;
							break;
					}
					if (!cont) break;
				}
				if ((*cpStr==' ' || *cpStr=='>' || *cpStr=='/') && !pingState) { // read the next cahr of the attribute value
					*cpTag = '\0';
					if (attr->mValue) free(attr->mValue);
					if ((attr->mValue=(char *)malloc(strlen(attrValue)+1))==NULL) continue;
					strcpy(attr->mValue, attrValue);
//					CleanStr(attr->mValue);
					elm->AddAttribute(attr);
					attr = new ElementAttribute();
					if (*cpStr==' ') {
						if (attrName) free(attrName);
						if ((attrName=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*attrName = '\0';
						cpTag = attrName;
						state = GET_ATTR_NAME_STATE;
						break;
					}
					if (*cpStr=='>') {
						if (tagValue) free(tagValue);
						if ((tagValue=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*tagValue = '\0';
						cpTag = tagValue;
						state = GET_TAG_VALUE_STATE;
						break;
					}
					if (*cpStr=='/' && *(cpStr+1)=='>') {
						cpStr++;
						state = TAG_END_STATE;
						break;
					}
				}
				*cpTag = *cpStr;
				cpTag++;
				break;

			case GET_TAG_VALUE_STATE: //Get tag value
				if (*cpStr=='<') {
					if (*(cpStr+1)=='!' && *(cpStr+2)=='-' && *(cpStr+3)=='-') {// This is a comment!
						oldState = state;
						state = TAG_COMMENT_STATE;
						break;
					}
					*cpTag = '\0',
					elm->SetValue(tagValue);
					if (*(cpStr+1)=='/') {
						cpStr++;
						if (endTagName) free(endTagName);
						if ((endTagName=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*endTagName = '\0';
						cpTag = endTagName;
						state = GET_END_TAG_NAME_STATE;
						break;
					}
					cpStr--;
					state = NEW_SUB_TAG_STATE;
					break;
				}
				*cpTag =*cpStr;
				cpTag++;
				break;
				
			case NEW_SUB_TAG_STATE: //Start to get sub tags from the actual tag
				if (*cpStr=='<') {
					if (*(cpStr+1)=='!' && *(cpStr+2)=='-' && *(cpStr+3)=='-') {
						oldState = state;
						state = TAG_COMMENT_STATE;
						break;
					}
					if (*(cpStr+1)=='/') {	
						if (endTagName) free(endTagName);
						if ((endTagName=(char *)malloc(strlen(cpStr)))==NULL) continue;
						*endTagName = '\0';
						cpTag = endTagName;
						cpStr++;
						state = GET_END_TAG_NAME_STATE;
						break;
					}
					subElm = new Element();
					elm->AddSubElement(subElm);
					if ((pStr=Parse(cpStr,subElm))==NULL) {
						cpStr = NULL;
						stop = 1;
						break;
					}
					cpStr = pStr;
					if (*cpStr=='<') {
						if (*(cpStr+1)=='!' && *(cpStr+2)=='-' && *(cpStr+3)=='-') {
							oldState = state;
							state = TAG_COMMENT_STATE;
							break;
						}
						noStep = 1;
					}
				}
				break;

			case GET_END_TAG_NAME_STATE: //Finish the actual tag
				if (*cpStr=='>') {
					state = TAG_END_STATE;
					break;
				}
				*cpTag = *cpStr;
				cpTag++;
				break;

			case TAG_END_STATE: //No more values, attrbute or subtag to read. Return!
				stop = 1;
				break;

			case TAG_COMMENT_STATE: //We read a comment but we do not save it!
				if (*cpStr=='-' && *(cpStr+1)=='-' && *(cpStr+2)=='>') {
					state = oldState;
					cpStr += 2;
				}
				break;

			case TAG_DECLARATION_STATE: //We read a declaraton but do not save it!
				if (*cpStr=='?' && *(cpStr+1)=='>') { //end of decleration
					state = oldState;
					cpStr++;
				}
				break;
		}
		if (noStep) noStep = 0;
		else if (cpStr!=NULL) cpStr++;
		else {
			stop = 1;
			break;
		}
	}
	if (tagName) free(tagName);
	if (tagValue) free(tagValue);
	if (attrName) free(attrName);
	if (attrValue) free(attrValue);
	if (endTagName) free(endTagName);
	if (attr) delete attr;

	if (stop) {
		if (cpStr==NULL) return NULL;
		cpStr--;
		return cpStr;
	}
	return NULL;
}//Parse

void MiniXML::Parse(char *xmlStr)
/**
 * Parse the given string. This create a new XML docoment tree for the class.
 */
{
	char *pharseStr, *subStr;
	Element *subElm;

	if ((pharseStr=CleanUpString(xmlStr))==NULL) return;
	DropTree();
	subElm = new Element();
	mXmlTree->AddSubElement(subElm);
	subStr = pharseStr;
	subStr = Parse(subStr, subElm);
	while (subStr) {
		subElm = new Element();
		mXmlTree->AddSubElement(subElm);
		subStr = Parse(subStr, subElm);
	}
	free(pharseStr);
}//Parse

void MiniXML::ParseFile(const char *fileName)
{
	FILE *fd;
	char *xmlStr, str[OUT_STR_BLOCK];
	int blocks;

	// Open and read the file into a string
	if ((fd=fopen(fileName,"r"))==NULL) return;
	if ((xmlStr=(char*)malloc(OUT_STR_BLOCK))==NULL) {
		fclose(fd);
		return;
	}
	blocks = 1;
	*xmlStr = '\0';
	while (fgets(str, OUT_STR_BLOCK, fd)) {
		if ((strlen(xmlStr)+strlen(str)+1) > (unsigned int)(blocks*OUT_STR_BLOCK)) {
			char *newStr;
			blocks++;
			if ((newStr=(char *)malloc(blocks*OUT_STR_BLOCK))==NULL) {
				fclose(fd);
				return;
			}
			strcpy(newStr, xmlStr);
			free(xmlStr);
			xmlStr = newStr;
		}
		strcat(xmlStr, str);
	}
	fclose(fd);

	// Parse the readed string and get the XML document tree
	Parse(xmlStr);
	free(xmlStr);
}//ParseFile

void MiniXML::InitOutStr()
/**
 * Prepare the output member string for new output.
 */
{
	if (mOutStr) free(mOutStr);
	if ((mOutStr=(char*)malloc(OUT_STR_BLOCK))==NULL) return;
	*mOutStr = '\0';
	mOutStrBlocks = 1;
	mFirstCall = 1;
}//InitOutStr

void MiniXML::AddOutStrBlocks(int b)
/**
 * If there is no more space int member output string this function will allocate more b new blocks to the string.
 */
{
	char *newStr;

	if ((newStr=(char *)malloc((mOutStrBlocks+b)*OUT_STR_BLOCK))==NULL) return;
	strcpy(newStr, mOutStr);
	if (mOutStr) free(mOutStr);
	mOutStr = newStr;
	mOutStrBlocks += b;
}//AddOutStrBlocks

void MiniXML::Tree2Str(Element *elm)
/**
 * Will cretate a whole XML document with, tags and attributes, based on the tree there start at the given Element.
 * The result can be found in the member output string.
 */
{
	ElementAttribute *attr;
	Element *subElm;
	char outStr[OUT_STR_BLOCK], header[256], *cp;

	if (elm==NULL) {
		return;
	}
	if (mFirstCall) strcpy(header,"<?xml version=\"1.0\"?>\n");
	if ((cp=elm->GetName())!=NULL) {
		if (mFirstCall) {
			sprintf(outStr,"%s<%s",header,cp);
			mFirstCall = 0;
		}
		else sprintf(outStr,"<%s",cp);
		attr = elm->GetAttributes();
		while (attr) {
			strcat(outStr," ");
			if (attr->mName!=NULL) {
				strcat(outStr, attr->mName);
				strcat(outStr,"=");
				if (attr->mValue!=NULL) {
					strcat(outStr,"\"");
					strcat(outStr,attr->mValue);
					strcat(outStr,"\"");
				}
			}
			attr = attr->mNext;
		}
//		strcat(outStr,">\n");
		strcat(outStr,">");
		if ((cp=elm->GetValue())!=NULL)	strcat(outStr, cp);
		else strcat(outStr, "\n");
//		strcat(outStr, "\n");
		if ((strlen(mOutStr)+strlen(outStr)+1) > (unsigned int)(mOutStrBlocks*OUT_STR_BLOCK)) AddOutStrBlocks(1);
		strcat(mOutStr, outStr);
	}

	subElm = elm->GetFirstSubElement();
	while (subElm) {
		Tree2Str(subElm);
		subElm = elm->GetNextSubElement();
	}
	if ((cp=elm->GetName())!=NULL) {
		sprintf(outStr,"</%s>\n",  cp);
		if ((strlen(mOutStr)+strlen(outStr)+1) > (unsigned int)(mOutStrBlocks*OUT_STR_BLOCK)) AddOutStrBlocks(1);
		strcat(mOutStr, outStr);	
	}
} //Tree2Str

