/***************************************************************************
                          ConnectionWrapper.cpp  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "ConnectionInterface.h"
#include "ConnectionWrapper.h"

/********** Thread used together with the Wrapper class **********/
#ifdef WIN32
DWORD connThread(void *connParams)
#elif defined(POSIX_THREAD)
void *connThread(void *connParams)
#endif
{
	void **cParams = (void**)connParams;
	ConnectionWrapper *connWrapper;
	ConnectionId *connId;
	char *params;
	int command, rVal, r;

	if (cParams==NULL) return NULL;
	connWrapper = (ConnectionWrapper *)cParams[0];
	connId = (ConnectionId *)cParams[1];
	if (connWrapper==NULL) return NULL;

	connWrapper->init(connId);
	cParams[0] = NULL;

	while (connWrapper->isConnThreadRunning()) {
		try {
			r = connWrapper->setEventWait();
			if (!connWrapper->isConnThreadRunning()) break;
			command = connWrapper->getConnCommand();

			connWrapper->setConnResp(NULL);
			switch (command) {
				case CONN_COMM_OPEN_CONNECTION:
					rVal = connWrapper->openConnection();
					connWrapper->setConnResp((void *)rVal);
					break;
				case CONN_COMM_CLOSE_CONNECTION:
					rVal = connWrapper->closeConnection();
					connWrapper->setConnResp((void *)rVal);
					break;
				case CONN_COMM_DO_SCRIPT_COMMAND:
					params = (char *)connWrapper->getConnParams();
					rVal = connWrapper->doScriptCommand(params);
					connWrapper->setConnResp((void *)rVal);
					break;
				case CONN_COMM_DISCONNECT:
					connWrapper->setDisconnectCloseTerminalWindow();
					rVal = connWrapper->closeConnection();
					break;
				case CONN_COMM_FREE_CHAR_POINTER:
					params = (char *)connWrapper->getConnParams();
					free(params);
					break;
			}

#ifdef POSIX_THREAD
			connWrapper->setEventLockMutex();
#endif
			r = connWrapper->respEventSet();
			if (command==CONN_COMM_DISCONNECT) connWrapper->respDiscSet();
		} catch (...) {
			printf("EXCEPTION:... inConnectionWrapper.conThread()\n");
			break;
		}
	}
	return 0;
}//connThread


/********** ConnectionWrapper class methods **********/


ConnectionWrapper::ConnectionWrapper()
{
	m_connIntf = NULL;
	tzset();
	m_connThreadRunning = FALSE;
	init (NULL);
}//ConnectionWrapper

ConnectionWrapper::ConnectionWrapper(ConnectionId *connId, int connThreadOn)
{
	m_connIntf = NULL;
	tzset();
	m_connThreadRunning = FALSE;
	if (connThreadOn) {
		void *threadParams[2];
		threadParams[0] = (void *)this;
		threadParams[1] = (void *)connId;
		if ((m_connThread = new ThreadClass(connThread, (void *)threadParams))==NULL) return;
		m_connThreadRunning = TRUE;
		int r = m_connThread->start();
		do {
			ThreadSleep(1);
		} while (threadParams[0]!=NULL);
	}
	else init (connId);
}//ConnectionWrapper

ConnectionWrapper::~ConnectionWrapper()
{
	if (m_connIntf) {
#ifdef USE_PERCOMM
		((PerCommConnection *)m_connIntf)->setDisconnectCloseTerminalWindow();
#elif defined(USE_X3270)
		((X3270Connection *)m_connIntf)->setDisconnectCloseTerminalWindow();
#elif defined(USE_EXEMUL)
		((ExEmulConnection *)m_connIntf)->setDisconnectCloseTerminalWindow();
#endif
		if (m_connThread) doDisconnect();
		else closeConnection();

		if (m_connThread) {
			m_connThreadRunning = FALSE;
			m_setEvent.set();
			ThreadSleep(1);
			m_connThread->kill();
			delete m_connThread;
			m_connThread = NULL;
			m_respEvent.set(); // Free threads there are waitin from a response from the thread we just killed
		}
#if defined(USE_PERCOMM)
		delete  (PerCommConnection *)m_connIntf;
#elif defined(USE_X3270)
		delete  (X3270Connection *)m_connIntf;
#elif defined (USE_EXEMUL)
		delete  (ExEmulConnection *)m_connIntf;
#endif
		m_connIntf = NULL;
	}
}//~ConnectionWrapper


void ConnectionWrapper::killConnectionWrapper()
{
	ThreadClass *tc = m_connThread;
	m_connThread = NULL;
	m_connThreadRunning = FALSE;
	if (tc) {
		tc->kill();
		delete tc;
		m_respEvent.set(); // Free threads there are waitin from a response from the thread we just killed
	}
}//killConnectionWrapper


void ConnectionWrapper::init(ConnectionId *connId)
{
#if defined(USE_PERCOMM)
	if (m_connIntf) delete (PerCommConnection *)m_connIntf;
	if ((m_connIntf = new PerCommConnection())==NULL) return;
	if (((PerCommConnection *)m_connIntf)->init()!=CI_NO_ERR) return;
#elif defined(USE_X3270)
	if (m_connIntf) delete (X3270Connection *)m_connIntf;
	if ((m_connIntf = new X3270Connection())==NULL) return;
#elif defined(USE_EXEMUL)
	if (m_connIntf) delete (ExEmulConnection *)m_connIntf;
	if ((m_connIntf = new ExEmulConnection())==NULL) return;
	if (((ExEmulConnection *)m_connIntf)->init()!=CI_NO_ERR) return;
#endif

	if (connId==NULL) {
#if defined(USE_PERCOMM)
		m_connId.sessionName = m_connId.connSessionName = 'A';
//		strcpy((char *)m_connId.emulCommand, "pcsws");
#elif defined(USE_X3270)
		strcpy(m_connId.fifoFileName,"/tmp/pipeFifo.1");
		strcpy(m_connId.emulCommand,"x3270 -script -model 2");
#elif defined(USE_EXEMUL)
		m_connId.sessionName = m_connId.connSessionName = 'A';
		strcpy((char *)m_connId.emulCommand, "Quick3270");
#endif
	}
	else memcpy(&m_connId, connId, sizeof(ConnectionId));

	m_connCommand = CONN_COMM_NO_COMMAND;
	m_connParams = NULL;
	m_connThread = NULL;
	m_disconnecting = FALSE;
}//init


int ConnectionWrapper::openConnection()
{
	int r;

	if (m_connIntf==NULL) {
#if defined(USE_PERCOMM)
		if ((m_connIntf = new PerCommConnection())==NULL) return CONN_COMM_MEMORY_ERROR;
		if ((r=((PerCommConnection *)m_connIntf)->init())!=CI_NO_ERR) return r;
#elif defined(USE_X3270)
		if ((m_connIntf = new X3270Connection())==NULL) return CONN_COMM_MEMORY_ERROR;
#elif defined(USE_EXEMUL)
		if ((m_connIntf = new ExEmulConnection())==NULL) return CONN_COMM_MEMORY_ERROR;
		if ((r=((ExEmulConnection *)m_connIntf)->init())!=CI_NO_ERR) return r;
#endif
	}
	return m_connIntf->openConnection(&m_connId);
}//openConnection


int ConnectionWrapper::closeConnection()
{
	int r = CI_NO_ERR;

	if (m_connIntf) r = m_connIntf->closeConnection(&m_connId);
	return r;
}//closeConnection


int ConnectionWrapper::setParameter(ConnParameter *param)
{
	int r = CI_NO_ERR;

	if (m_connIntf) r = m_connIntf->setParameter(param);
	return r;
}//setParameter


int ConnectionWrapper::getParameter(ConnParameter *param)
{
	int r = CI_NO_ERR;

	if (m_connIntf) r = m_connIntf->getParameter(param);
	return r;
}//getParameter


ConnParameter **ConnectionWrapper::getAllOutputParameters(char *scriptName)
{
	ConnParameter **r = NULL;

	if (m_connIntf) r = m_connIntf->getAllOutputParameters(scriptName);
	return r;
}//getAllOutputParameters


int ConnectionWrapper::doScriptCommand(char *command)
{
	int r = CI_NO_ERR;

	if (m_connIntf) r = m_connIntf->doScriptCommand(command);
	return r;
}//getVarible


void ConnectionWrapper::parseScriptFile(char *filename)
{
	if (m_connIntf) m_connIntf->parseScriptFile(filename);
}//parseScriptFile

		
BOOL ConnectionWrapper::isConnected()
{
	if (m_connIntf) return (BOOL)m_connIntf->isConnected();
	else return FALSE;
}//isConnected

void *ConnectionWrapper::doCommand(int command, void *params)
{
	if (m_disconnecting) return NULL;
	if (m_connThreadRunning) {
		m_connCommand = command;
		m_connParams = params;
		m_setEvent.set();
		m_respEvent.wait();
	}
	else doCommandNoThread(command, params);
	return m_connResp;
}//doCommand

void ConnectionWrapper::doDisconnect()
{
	int r;

	m_disconnecting = TRUE;
	m_connCommand = CONN_COMM_DISCONNECT;
	if (m_connThreadRunning) {
		r = m_setEvent.set();
		m_discEvent.wait();
	}
	else doCommandNoThread(m_connCommand, NULL);
}//doDisconnect


void ConnectionWrapper::doCommandNoThread(int command, void *params)
{
	char *cParam;
	int rVal;

printf("doCommandNoThread(%d,...)\n",command);
	switch (command) {
		case CONN_COMM_OPEN_CONNECTION:
			rVal = openConnection();
			m_connResp = (void *)rVal;
			break;
		case CONN_COMM_CLOSE_CONNECTION:
			rVal = closeConnection();
			m_connResp = (void *)rVal;
			break;
		case CONN_COMM_DO_SCRIPT_COMMAND:
			cParam = (char *)params;
			rVal = doScriptCommand(cParam);
			m_connResp = (void *)rVal;
			break;
		case CONN_COMM_DISCONNECT:
			setDisconnectCloseTerminalWindow();
			rVal = closeConnection();
			break;
		case CONN_COMM_FREE_CHAR_POINTER:
			free(params);
			break;
	}
}//doCommandNoThread

