/***************************************************************************
                          ExEmulConnection.cpp  -  description
                             -------------------
    begin                : Tue Jul 22 2003
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef USE_EXEMUL

#include <stdio.h>
#define NO_EXTERN
#include "ExEmulConnection.h"
#undef NO_EXTERN
#include "ThreadIntefaceWin32.h"
#include "ThreadIntefacePthread.h"

char HllapiFileName[_MAX_PATH+1];
char PcsapiFileName[_MAX_PATH+1];
//int Use32bitParamHllapi = 1;
int Use32bitParamHllapi = 0;
MutexClass pcsapiMutex;

ExEmulConnection::ExEmulConnection()
{
	m_connectScriptDone = 0;
	m_closeTerminalWindow = 0;
	m_haveEmulatorOpen = 0;
	m_connected = 0;
	m_scriptfileParsed = 0;
	*m_symbolicConnName = '\0';
	m_hinstHllapi = 0;
	m_hllapi16 = NULL;
	m_hllapi32 = NULL;
	m_connSessionName = ' ';
	memset(&m_emulProcessInformation, 0, sizeof(PROCESS_INFORMATION));
}//ExEmulConnection


ExEmulConnection::~ExEmulConnection()
{
	if ((UINT) m_hinstHllapi > 32) FreeLibrary(m_hinstHllapi);
	m_hllapi16 = NULL;
	m_hllapi32 = NULL;
}//~ExEmulConnection


int ExEmulConnection::init()
{
	if ((UINT) m_hinstHllapi <= 32) {

		m_hinstHllapi = LoadLibraryEx(HllapiFileName,NULL,0);
		if ((UINT) m_hinstHllapi > 32) {
			if (Use32bitParamHllapi) {
				if ((m_hllapi32=(void (WINAPI *)(long *,char *,long *,long *))GetProcAddress(m_hinstHllapi, "hllapi"))==NULL) {
					if ((m_hllapi32=(void (WINAPI *)(long *,char *,long *,long *))GetProcAddress(m_hinstHllapi, "HLLAPI"))==NULL) return CI_ERR_NOHLLAPIFUNC;
				}
			}
			else {
				if ((m_hllapi16=(void (WINAPI *)(short *,char *,short *,short *))GetProcAddress(m_hinstHllapi, "hllapi"))==NULL) {
					if ((m_hllapi16=(void (WINAPI *)(short *,char *,short *,short *))GetProcAddress(m_hinstHllapi, "HLLAPI"))==NULL) return CI_ERR_NOHLLAPIFUNC;
				}
			}
		}
		else return CI_ERR_OPENHLLAPI;
	}
	return CI_NO_ERR;
}//init


int ExEmulConnection::openConnection(ConnectionId *id)
{
	char s[1];
	long l, r;

	if (id==NULL) return CI_ERR_NULL_CONN_ID;

	strncpy(m_symbolicConnName, id->symbolicConnName, 65);
	m_symbolicConnName[64] = '\0';

	if (!m_haveEmulatorOpen) {
		if ((r=openTerminalWindow(id))!=HARC_SUCCESS) return r;
	}

	id->connSessionName = 'A';
/*	if (id->sessionName == '*') { //try to find a free session
		for(s[0]='A'; s[0]<='Z'; s[0]++) {
			l = 1;
			r = 0;
			callHllApi(HA_CONNECT_PS, s, &l, &r, FALSE);//Connet Presentation Space
			if (r == HARC_SUCCESS) break;
		}
	}
	else {*/
		l = 1;
		s[0] = id->sessionName;
		r = 0;
		testConnection(s[0], &r);
//	}
	if (r!=HARC_SUCCESS) {
		if (m_haveEmulatorOpen) closeTerminalWindow(id);
		else return r;
	}
	id->connSessionName = s[0];
	m_connSessionName = s[0];
	if (id->sessionName == '*') id->sessionName = s[0];
	m_connected = 1;

	if (*id->scriptsFileName=='\0') return CI_NO_ERR;
	if (!m_scriptfileParsed) {
		m_miniXML.DropTree();
		m_miniXML.ParseFile(id->scriptsFileName);
	}

	if ((r=firstSteps())!=HARC_SUCCESS) return r;
	return CI_NO_ERR;
}//openConnection


int ExEmulConnection::closeConnection(ConnectionId *id)
{
	char s[1];
	long l, r;

	if (id==NULL) return CI_ERR_NULL_CONN_ID;

	if (!m_connected) {
		if (m_haveEmulatorOpen && m_closeTerminalWindow) closeTerminalWindow(id);
		return CI_NO_ERR;
	}
	r = 0;
	
	r = lastSteps();
	s[0] = id->connSessionName;
	l = 1;
	r = 0;
	callHllApi(HA_DISCONNECT_PS, s, &l, &r, FALSE);//Disconnet presentation space
	if (m_closeTerminalWindow) {
		closeTerminalWindow(id);
	}
	m_connSessionName = ' ';
	m_connected = 0;
	return CI_NO_ERR;
}//closeConnection


int ExEmulConnection::setParameter(ConnParameter *param)
{
	Element *xmlTree, *subTree, *varTag;
	char *attrVal;

	if (param==NULL) return HARC_BAD_PARM;
	if (param->scriptName==NULL || param->paramName==NULL || param->value==NULL) return HARC_BAD_PARM;

	if ((xmlTree=m_miniXML.GetTree())==NULL) return HARC_BAD_PARM;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return HARC_BAD_PARM;

	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return HARC_BAD_PARM;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL) {
		if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->paramName)==0) {
			// Validate the direction
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"out")==0) return HARC_BAD_PARM;
			varTag->SetValue(param->value);
			return CI_NO_ERR;
		}
		varTag = varTag->GetNextElement();
	}

	return HARC_BAD_PARM;
}//setParameter


int ExEmulConnection::getParameter(ConnParameter *param)
{
	Element *xmlTree, *subTree, *varTag;
	char *attrVal;

	if (param==NULL) return HARC_BAD_PARM;
	if (param->scriptName==NULL || param->paramName==NULL) return HARC_BAD_PARM;

	if ((xmlTree=m_miniXML.GetTree())==NULL)  return HARC_BAD_PARM;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return HARC_BAD_PARM;

	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return HARC_BAD_PARM;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL ) {
		if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->paramName)==0) {
			// Validate the direction
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"in")==0) return HARC_BAD_PARM;
			if (param->value) free(param->value);
			param->setValue(varTag->GetValue());
			return CI_NO_ERR;
		}
		varTag = varTag->GetNextElement();
	}

	return HARC_BAD_PARM;
}//getParameter


ConnParameter **ExEmulConnection::getAllOutputParameters(char *scriptName)
{
	Element *xmlTree, *subTree, *varTag;
	char *varName, *attrVal;
	ConnParameter **params, *param;
	int maxParams = 10;
	int paramInx, i;

	if (scriptName==NULL) return NULL;


	if ((xmlTree=m_miniXML.GetTree())==NULL)  return NULL;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return NULL;

	if ((params = new ConnParameter *[maxParams])==NULL) return NULL;
	paramInx = 0;
	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return NULL;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL ) {
		if ((varName=varTag->GetAttrubuteValue("name"))!=NULL) {
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"in")==0) continue;
			if ((param = new ConnParameter())==NULL) continue;
			param->setScriptName(scriptName);
			param->setParamName(varName);
			param->setValue(varTag->GetValue());
			params[paramInx] = param;
			paramInx++;
			if (paramInx == (maxParams-1)) {
				ConnParameter **newParams;
				if ((newParams = new ConnParameter *[maxParams+10])==NULL) continue;
				for (i=0; i<maxParams; i++) {
					newParams[i] = params[i];
				}
				delete []params;
				params = newParams;
				maxParams += 10;
			}
		}
		varTag = varTag->GetNextElement();
	}
	params[paramInx] = NULL;

	
	return params;
}//getAllOutputParameters


int ExEmulConnection::parseCommands(Element *dataTree, Element *commTree)
{
	Element  *command, *subTag, *varTag;
	long p, l, r;
	char *tagName, *tagVal, *varName, ps[1921], *psPtr, *attrVal;

	if (commTree==NULL) return HARC_BAD_PARM;

	command = commTree->GetFirstSubElement();
	while (command!=NULL) {
		if ((tagName=command->GetName())==NULL) {
			command = commTree->GetNextSubElement();
			continue;
		}
		tagVal = command->GetValue();

		if (strcmp(tagName,"send-str")==0) {
			// do we have to send a variable or fixed string?
			if ((subTag=command->GetFirstSubElement())!=NULL) {
				if (strcmp(subTag->GetName(),"param-ref")==0) { //Send a variable
					if (dataTree==NULL) continue;
					// Find the name of the variable
					varName=subTag->GetAttrubuteValue("name");
					if (varName==NULL) continue;
					// Find the variable
					varTag = dataTree->GetFirstSubElement();
					while (varTag!=NULL) {
						if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, varName)==0) {
							tagVal = varTag->GetValue();
							if (tagVal!=NULL) r = sendStrAsKeys(tagVal, strlen(tagVal));
							break;
						}
						varTag = varTag->GetNextElement();
					}
				}
			}
			else {//Send fixed string
				if (tagVal!=NULL) r = sendStrAsKeys(tagVal, strlen(tagVal));
			}
		}
		else if (strcmp(tagName,"get-str")==0) {
			if ((r=copyPs2Str(ps, 1920))==HARC_SUCCESS) {
				p = l = 0;
				if ((attrVal=command->GetAttrubuteValue("pos"))!=NULL) {
					p = atol(attrVal);
					if (p > 1920) p = 1920;
				}
				if ((attrVal=command->GetAttrubuteValue("len"))!=NULL) {
					l = atol(attrVal);
					if ((p+l) > 1920) l = 1920-p;
				}
				psPtr = &ps[p-1];
				psPtr[l] = '\0';
				if ((subTag=command->GetFirstSubElement())!=NULL) {
					if (strcmp(subTag->GetName(),"param-ref")==0) { //get the variable name
						if (dataTree==NULL) continue;
						// Find the name of the variable
						varName=subTag->GetAttrubuteValue("name");
						if (varName==NULL) continue;
						// Find the variable
						varTag = dataTree->GetFirstSubElement();
						while (varTag!=NULL) {
							if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, varName)==0) {
								varTag->SetValue(psPtr);
								break;
							}
							varTag = varTag->GetNextElement();
						}
					}
				}
			}
		}
		else if (strcmp(tagName,"set-cursor")==0) {
			l = atol(tagVal);
			r = setCursor(l);
		}
		else if (strcmp(tagName,"send-clear-key")==0) {
			r = sendClearKey();
		}
		else if (strcmp(tagName,"send-reset-key")==0) {
			r = sendResetKey();
		}
		else if (strcmp(tagName,"send-enter-key")==0) {
			r = sendEnterKey();
		}
		else if (strcmp(tagName,"send-pa-key")==0) {
			l = atol(tagVal);
			r = sendPaKey(l);
		}
		else if (strcmp(tagName,"send-tab-key")==0) {
			r = sendTabKey();
		}
		else if (strcmp(tagName,"send-backtab-key")==0) {
			r = sendBacktabKey();
		}
		else if (strcmp(tagName,"send-pf-key")==0) {
			l = atol(tagVal);
			r = sendPfKey(l);
		}
		else if (strcmp(tagName,"send-home-key")==0) {
			r = sendHomeKey();
		}
		else if (strcmp(tagName,"send-cursor-left-key")==0) {
			r = sendCursorLeftKey();
		}
		else if (strcmp(tagName,"send-cursor-right-key")==0) {
			r = sendCursorRightKey();
		}
		else if (strcmp(tagName,"send-cursor-up-key")==0) {
			r = sendCursorUpKey();
		}
		else if (strcmp(tagName,"send-cursor-down-key")==0) {
			r = sendCursorDownKey();
		}
		else if (strcmp(tagName,"wait")==0) {
			l = atol(tagVal);
			ThreadSleep(l);
		}
		if (r!=HARC_SUCCESS) return r;

		command = commTree->GetNextSubElement();
	}

	return HARC_SUCCESS;
}//parseCommands


int ExEmulConnection::parse(char *scriptName)
{
	Element *xmlTree, *dataTree, *firstStep, *subTree, *step, *commands, *response;
	long r, stop;
	char *cp, *nextStepName, ps[1921], *attrVal;

	if ((xmlTree=m_miniXML.GetTree())==NULL)  return HARC_BAD_PARM;
	xmlTree->InitSearch();
	if ((subTree=xmlTree->SearchFor("scripts"))==NULL) return HARC_BAD_PARM;
	subTree->InitSearch();
	step = subTree->SearchFor("script");
	while (step!=NULL) {
		if ((attrVal=step->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal,scriptName)==0) {
			break;
		}
		step = subTree->SearchFor("script");
	}
	if (step==NULL) return HARC_BAD_PARM;
	subTree = step;

	subTree->InitSearch();
	dataTree = subTree->SearchFor("params");
	if ((firstStep=subTree->SearchFor("first-step"))==NULL) return HARC_BAD_PARM;
	if ((nextStepName=firstStep->GetValue())==NULL) return HARC_BAD_PARM;
	stop = 0;
	while (!stop) {
		if (nextStepName==NULL) return HARC_SYSTEM_ERROR;
		if (strcmp(nextStepName,"stop")==0) {
			stop = 1;
			break;
		}
		if (strcmp(nextStepName,"error")==0) {
			stop = 1;
			return HARC_SYSTEM_ERROR;
		}
		//Find the step
		subTree->InitSearch();
		do  {
			if ((step=subTree->SearchFor("step"))!=NULL) {
				if ((attrVal=step->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal,nextStepName)==0) {
					break;
				}
			}
		} while (step!=NULL);
		if (step==NULL) return HARC_BAD_PARM;

		//Do the commands
		step->InitSearch();
		commands=step->SearchFor("commands");
		if ((r=parseCommands(dataTree, commands))!=HARC_SUCCESS) return r;

		//Find next step
		if ((r=copyPs2Str(ps,1920))!=HARC_SUCCESS) return r;
		ps[1920] = '\0';
		step->InitSearch();
		if ((firstStep=step->SearchFor("next"))!=NULL) nextStepName = nextStepName=firstStep->GetValue();
		else nextStepName = NULL;
		step->InitSearch();
		response=step->SearchFor("response");
		while (response!=NULL) {
			if ((cp=response->GetValue())!=NULL) {
				if (strstr(ps,cp) != NULL) {
					if ((attrVal=response->GetAttrubuteValue("next"))!=NULL) {
						nextStepName = attrVal;
						break;
					}
				}
			}
			response=step->SearchFor("response");
		}
	}

	return HARC_SUCCESS;
}//parse


void ExEmulConnection::callHllApiRaw(long funcNum, char *dataString, long *dataLength, long *returnCode)
{
	short f16, l16, r16;
	long f32, l32, r32;
	char *s, ps[2];

	s = dataString;
	ps[0] = m_connSessionName;

	if (Use32bitParamHllapi) {
		if (!m_hllapi32) {
			*returnCode = CI_ERR_NOHLLAPIFUNC;
			return;
		}
		f32 = funcNum;
		l32 = *dataLength;
		r32 = *returnCode;
//		Hllapi32(&f32, s, &l32, &r32);
		m_hllapi32(&f32, s, &l32, &r32); //Call HLLAPI function
		*dataLength = l32;
		*returnCode = r32;
	}
	else {
		if (!m_hllapi16) {
			*returnCode = CI_ERR_NOHLLAPIFUNC;
			return;
		}
		f16 = (short)funcNum;
		l16 = (short)*dataLength;
		r16 = (short)*returnCode;
//		Hllapi16(&f16, s, &l16, &r16);
		m_hllapi16(&f16, s, &l16, &r16);
		*dataLength = (long) l16;
		*returnCode = (long) r16;
	}
}//callHllApiRaw


void ExEmulConnection::callHllApi(long funcNum, char *dataString, long *dataLength, long *returnCode, BOOL withConnectPS)
{
	long l, r;
	char *s, ps[2];

	s = dataString;
	ps[0] = m_connSessionName;

pcsapiMutex.lock(); //As PC somtime create a dead lock with HLLAPI and PCSAPI32
	if (withConnectPS) {
		l = 1;
		r = 0;
		callHllApiRaw(HA_CONNECT_PS, ps, &l, &r);//Connet Presentation Space
		if (r != HARC_SUCCESS) {
			*returnCode = r;
			pcsapiMutex.unlock();
			return;
		}
	}

	l = *dataLength;
	r = *returnCode;
	callHllApiRaw(funcNum, s, &l, &r); //Call HLLAPI function
	*dataLength = l;
	*returnCode = r;

	if (withConnectPS) {
		l = 1;
		r = 0;
		callHllApiRaw(HA_DISCONNECT_PS, ps, &l, &r);//Disconnet Presentation Space
		if (r != HARC_SUCCESS) {
				if (*returnCode==HARC_SUCCESS) *returnCode = r;
		}
	}
pcsapiMutex.unlock();
}//callHllApi


void ExEmulConnection::testConnection(char psName, long *returnCode)
{
	long l, r;
	char ps[2];

	ps[0] = psName;

pcsapiMutex.lock(); //As PC somtime create a dead lock with HLLAPI and PCSAPI32
	l = 1;
	r = 0;
	callHllApiRaw(HA_CONNECT_PS, ps, &l, &r);//Connet Presentation Space
	*returnCode = r;

	l = 1;
	r = 0;
	callHllApiRaw(HA_DISCONNECT_PS, ps, &l, &r);//Disconnet Presentation Space
	if (r != HARC_SUCCESS) {
		if (*returnCode==HARC_SUCCESS) *returnCode = r;
	}
pcsapiMutex.unlock();
}//testConnection


int ExEmulConnection::hllapiWait(int timeout)
{
	long l, r;
	char s[2];
	int secCount;

	secCount = 0;
	do {
		l = r = 0;
		callHllApi(HA_WAIT, s, &l, &r, TRUE);//Wait
		if (r != HARC_SUCCESS) {
			if (r!=HARC_BUSY) return r;
			else if (timeout < 0x7FFF) {
				ThreadSleep(1);
				secCount++;
				if (secCount > timeout) return CI_ERR_TIMEOUT;
			}
			else ThreadSleep(1);
		}
	} while (r != HARC_SUCCESS);
	return r;
}//hllapiWait


int ExEmulConnection::firstSteps()
{
	long r;

	if (!m_connectScriptDone) {
		if ((r=parse("connect"))!=HARC_SUCCESS) return r;
		m_connectScriptDone = 1;
	}

	return HARC_SUCCESS;
}//firstSteps


int ExEmulConnection::lastSteps()
{
	long r;

	r=parse("disconnect");
	m_connectScriptDone = 0;
	return r;
}//lastSteps


int ExEmulConnection::sendStrAsKeys(char *data, long length)
{
	char s[2], *cp;;
	long r;

	cp = data;
	for (int i=0; i<length; i++) {
		s[0] = *cp;
		if((r=sendKey(s, 1, 0x7FFF))!=HARC_SUCCESS) return r;
		cp++;
	}
	return HARC_SUCCESS;
}//sendStrAsKeys


int ExEmulConnection::copyPs2Str(char *data, long maxLength)
{
	long l, r;

	l = maxLength;
	r = 1;
	callHllApi(HA_COPY_PS_TO_STR, data, &l, &r, TRUE);
	if (r != HARC_SUCCESS) return r;
	if (hllapiWait(0x7FFF) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//copyPs2Str


int ExEmulConnection::copyStr2Ps(char *data, long length)
{
	long l, r;

	l = length;
	r = 1;
	callHllApi(HA_COPY_STR_TO_PS, data, &l, &r, TRUE);
	if (r != HARC_SUCCESS) {
		if (r!=HARC_TRUNCATION) return r;
	}
	if (hllapiWait(0x7FFF) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//copyStr2Ps


int ExEmulConnection::copyStr2Ps(char *data, long length, long pos)
{
	long l, r;

	l = length;
	r = pos;
	callHllApi(HA_COPY_STR_TO_PS, data, &l, &r, TRUE);
	if (r != HARC_SUCCESS) {
		if (r!=HARC_TRUNCATION) return r;
	}
	if (hllapiWait(0x7FFF) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//copyStr2Ps


int ExEmulConnection::copyStr2Field(char *data, long length, long pos)
{
	long l, r;

	l = length;
	r = pos;
	callHllApi(HA_COPY_STR_TO_FIELD, data, &l, &r, TRUE);
	if (r != HARC_SUCCESS) return r;
	if (hllapiWait(0x7FFF) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//copyStr2Field


int ExEmulConnection::setCursor(long pos)
{
	char c[1];
	long l, r;

	l = 0;
	r = pos;
	callHllApi(HA_SET_CURSOR, c, &l, &r, TRUE);
	if (r != HARC_SUCCESS) return r;
	if (hllapiWait(0x7FFF) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//setCursor


int ExEmulConnection::sendKey(char *data, long length, int wait)
{
	long l, r;

	l = length;
	r = 0;
	callHllApi(HA_SENDKEY, data, &l, &r, TRUE);
	if (r != HARC_SUCCESS) return r;
	if (hllapiWait(wait) != HARC_SUCCESS) return HARC_BUSY;
	return HARC_SUCCESS;
}//sendKey


int ExEmulConnection::sendClearKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'C';
	//Send Key (clear)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendClearKey


int ExEmulConnection::sendResetKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'R';
	//Send Key (clear)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendResetKey


int ExEmulConnection::sendEnterKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'E';
	//Send Key (Enter)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendEnterKey


int ExEmulConnection::sendEnterKey(int wait)
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'E';
	//Send Key (Enter)
	if ((r=sendKey(s, 2, wait)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendEnterKey


int ExEmulConnection::sendPaKey(int key)
{
	char s[2];
	long r;

	s[0] = '@';
	switch (key) {
		case 1: s[1] = 'x'; break;
		case 2: s[1] = 'y'; break;
		case 3: s[1] = 'z'; break;
		default:
			return HARC_BAD_PARM;
	}

	//Send Key (PA key)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendPa1Key


int ExEmulConnection::sendTabKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'T';
	//Send Key (Tab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendTabKey


int ExEmulConnection::sendBacktabKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'B';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendBacktabKey


int ExEmulConnection::sendPfKey(int key)
{
	char s[2];
	long r;

	s[0] = '@';
	switch (key) {
		case 1: s[1] = '1'; break;
		case 2: s[1] = '2'; break;
		case 3: s[1] = '3'; break;
		case 4: s[1] = '4'; break;
		case 5: s[1] = '5'; break;
		case 6: s[1] = '6'; break;
		case 7: s[1] = '7'; break;
		case 8: s[1] = '8'; break;
		case 9: s[1] = '9'; break;
		case 10: s[1] = 'a'; break;
		case 11: s[1] = 'b'; break;
		case 12: s[1] = 'c'; break;
		case 13: s[1] = 'd'; break;
		case 14: s[1] = 'e'; break;
		case 15: s[1] = 'f'; break;
		case 16: s[1] = 'g'; break;
		case 17: s[1] = 'h'; break;
		case 18: s[1] = 'i'; break;
		case 19: s[1] = 'j'; break;
		case 20: s[1] = 'k'; break;
		case 21: s[1] = 'l'; break;
		case 22: s[1] = 'm'; break;
		case 23: s[1] = 'n'; break;
		case 24: s[1] = 'o'; break;
		default:
			return HARC_BAD_PARM;
	}
	//Send Key (PF key)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendPfKey


int ExEmulConnection::sendHomeKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = '0';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendHomeKey


int ExEmulConnection::sendCursorLeftKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'L';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendCursorLeftKey


int ExEmulConnection::sendCursorRightKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'Z';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendCursorRightKey


int ExEmulConnection::sendCursorUpKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'U';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendCursorUpKey


int ExEmulConnection::sendCursorDownKey()
{
	char s[2];
	long r;

	s[0] = '@';
	s[1] = 'V';
	//Send Key (Backtab)
	if ((r=sendKey(s, 2, 0x7FFF)) != HARC_SUCCESS) return r;
	return HARC_SUCCESS;
}//sendCursorDownKey


int ExEmulConnection::openTerminalWindow(ConnectionId *id)
{
	STARTUPINFO startupInfo;
	int r = HARC_SUCCESS;

	if (!m_haveEmulatorOpen && *id->emulCommand!='\0') {
		if (id->sessionName=='*') id->connSessionName = ' ';
		else id->connSessionName = id->sessionName;

		memset(&startupInfo, 0, sizeof(STARTUPINFO));
		startupInfo.cb = sizeof(STARTUPINFO);
		startupInfo.dwFlags = STARTF_USESHOWWINDOW;
		startupInfo.wShowWindow = SW_SHOW;

		memset(&m_emulProcessInformation, 0, sizeof(PROCESS_INFORMATION));

//		if (CreateProcess(NULL, id->wsProfileFileName, NULL, NULL, FALSE, DETACHED_PROCESS, NULL, NULL, &startupInfo, &m_emulProcessInformation)) {
		if (CreateProcess(NULL, id->emulCommand, NULL, NULL, FALSE, 0, NULL, NULL, &startupInfo, &m_emulProcessInformation)) {
			Sleep(5000); //Sleep 3 sec. so the PC system can recover
			ThreadSleep(id->timeOutSec);
		}
		else {
			r = GetLastError();
printf("Error occured while creating process (%c): %d\n",id->connSessionName,r);
			closeTerminalWindow(id);
//			return CI_ERR_CONNECT;
			return r;
		}
		m_haveEmulatorOpen = 1;
		m_closeTerminalWindow = 0;
	}
	return HARC_SUCCESS;
}//openTerminalWindow


int ExEmulConnection::closeTerminalWindow(ConnectionId *id)
{
	HANDLE hProc;
	int r = HARC_SUCCESS;

//	r = CloseHandle(m_emulProcessInformation.hThread);
//	r = r && CloseHandle(m_emulProcessInformation.hProcess);
//	r = r && TerminateProcess(m_emulProcessInformation.hProcess, 0);
//	r = TerminateProcess(m_emulProcessInformation.hProcess, 0);
	if ((hProc=OpenProcess(SYNCHRONIZE|PROCESS_TERMINATE, FALSE, m_emulProcessInformation.dwProcessId))==NULL) {
		printf("closeTerminalWindow(). Does not have PROCESS_TERMINATE right for the emulatore window\n");
	}
	else r = TerminateProcess(hProc, 0);
//	PostMessage(hwnd, WM_CLOSE, 0, 0) ;

	if (r) {
		Sleep(5000); //Sleep 3 sec. so the PC system can recover
	}
	else {
		r = GetLastError();
	}
	memset(&m_emulProcessInformation, 0, sizeof(PROCESS_INFORMATION));
	m_haveEmulatorOpen = 0;
	m_closeTerminalWindow = 0;
	m_connected = 0;
	return r;
}//closeTerminalWindow


int ExEmulConnection::doScriptCommand(char *command)
{
	return parse(command);
}//doScriptCommand


void ExEmulConnection::parseScriptFile(char *filename)
{
	m_miniXML.DropTree();
	m_miniXML.ParseFile(filename);
	m_scriptfileParsed = 1;
}

#endif
