/***************************************************************************
                          ThreadIntefacePthread.cpp  -  description
                             -------------------
    begin                : Tue May 14 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef POSIX_THREAD

#include <unistd.h>
#include <signal.h>
#define LOCAL_INCLUDE
#include "ThreadIntefacePthread.h"

MutexClass::MutexClass()
{
	pthread_mutexattr_t mutex_attr;
	
	pthread_mutexattr_init(&mutex_attr);
	pthread_mutex_init(&m_mutex, &mutex_attr);
	pthread_mutexattr_destroy(&mutex_attr);
}//MutexClass

MutexClass::~MutexClass()
{
	pthread_mutex_destroy(&m_mutex);
}

int MutexClass::lock()
{
	return pthread_mutex_lock(&m_mutex);
}//lock

int MutexClass::unlock()
{
	return pthread_mutex_unlock(&m_mutex);
}//unlock



Event::Event()
{
	pthread_mutexattr_t mutex_attr;
	pthread_condattr_t cond_attr;
	
	pthread_mutexattr_init(&mutex_attr);
	pthread_mutex_init(&m_cond_mutex, NULL);
	
	pthread_condattr_init(&cond_attr);
	pthread_cond_init(&m_cond, &cond_attr);
	
	pthread_mutexattr_destroy(&mutex_attr);
	pthread_condattr_destroy(&cond_attr);
	m_isMutextLocked = 0;
}//Event

Event::~Event()
{
	pthread_cond_destroy(&m_cond);
	pthread_mutex_destroy(&m_cond_mutex);
}//~Event

int Event::wait()
{
	int r;
	if (!m_isMutextLocked) pthread_mutex_lock(&m_cond_mutex);
	r = pthread_cond_wait(&m_cond, &m_cond_mutex);
	pthread_mutex_unlock(&m_cond_mutex);
	m_isMutextLocked = 0;
	return r;
}//wait

int Event::set()
{
	int r;

	pthread_mutex_lock(&m_cond_mutex);
	r = pthread_cond_signal(&m_cond);
	pthread_mutex_unlock(&m_cond_mutex);
	m_isMutextLocked = 0;
	return r;
}//set

int Event::lockEventMutext()
{
	if (m_isMutextLocked) return 0;
	pthread_mutex_lock(&m_cond_mutex);
	m_isMutextLocked = 1;
}//lockEventMutext





void *runThreadClassThread(void *pObj)
{
	ThreadClass *pThread = (ThreadClass *)pObj;

	pThread->callStartRutine();

	pthread_exit(NULL);
	return NULL;
}//runThreadClassThread



ThreadClass::ThreadClass(void *(*startRutine)(void *), void *arg)
{
	m_startRutine = startRutine;
	m_threadArg = arg;
	m_threadId = 0;
	m_doNotKillMe = 0;
}//ThreadClass

ThreadClass::~ThreadClass()
{
	if (m_threadId && !m_doNotKillMe) pthread_kill(m_threadId, SIGKILL);
}//~ThreadClass

int ThreadClass::start()
{
	pthread_attr_t threadAttr;
	int r;
	
	if ((r = pthread_attr_init(&threadAttr))!=0) return r;
	if ((r = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED))!=0) return r;
	if ((r = pthread_create(&m_threadId, &threadAttr, runThreadClassThread, (void *)this))!=0) return r;
	pthread_attr_destroy(&threadAttr);
	return r;
}//start

void ThreadClass::kill()
{
	if (m_threadId) pthread_kill(m_threadId, SIGKILL);
	m_threadId = 0;
}//kill

void *ThreadClass::callStartRutine()
{
	if (m_startRutine) return  m_startRutine(m_threadArg);
	return NULL;
}//callStartRutine

void ThreadSleep(int sec)
{
	sleep(sec);
}//ThreadSleep

#endif
