/***************************************************************************
                          ThreadIntefaceWin32.cpp  -  description
                             -------------------
    begin                : Tue May 14 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef WIN32

#include <signal.h>
#define LOCAL_INCLUDE
#include "ThreadIntefaceWin32.h"


MutexClass::MutexClass()
{
	InitializeCriticalSection(&m_mutex);
}//MutexClass

MutexClass::~MutexClass()
{
	DeleteCriticalSection(&m_mutex);
}

int MutexClass::lock()
{
	EnterCriticalSection(&m_mutex);
	return 0;
}//lock

int MutexClass::unlock()
{
	LeaveCriticalSection(&m_mutex);
	return 0;
}//unlock




Event::Event()
{
	m_hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
}//Event

Event::~Event()
{
	if (m_hEvent!=0) CloseHandle(m_hEvent);
}//~Event

int Event::wait()
{
	if (m_hEvent==0) return 0;
	WaitForSingleObject(m_hEvent, INFINITE);
	ResetEvent(m_hEvent);
	return 0;
}//wait


int Event::set()
{
	if (m_hEvent==0) return 0;
	return SetEvent(m_hEvent);
}//set





DWORD runThreadClassThread(void *pObj)
{
	ThreadClass *pThread = (ThreadClass *)pObj;

	pThread->callStartRutine();

	ExitThread(0);
	return 0;
}//runThreadClassThread



ThreadClass::ThreadClass(DWORD (*startRutine)(void *), void *arg)
{
	m_startRutine = startRutine;
	m_threadArg = arg;
	m_threadId = 0;
	m_threadH = 0;
	m_doNotKillMe = 0;
}//ThreadClass

ThreadClass::~ThreadClass()
{
	if (m_threadH && !m_doNotKillMe) {
		TerminateThread(m_threadH, 0);
		CloseHandle(m_threadH);
	}
}//~ThreadClass

int ThreadClass::start()
{
	if ((m_threadH=CreateThread(0, 0, (LPTHREAD_START_ROUTINE) runThreadClassThread, (void *)this, 0, &m_threadId))==NULL) return 1;
	return 0;
}//start

void ThreadClass::kill()
{
	if (m_threadH) CloseHandle(m_threadH);
	m_threadId = 0;
	m_threadH = 0;
}//kill

void *ThreadClass::callStartRutine()
{
	if (m_startRutine) m_startRutine(m_threadArg);
	return NULL;
}//callStartRutine




void ThreadSleep(int sec)
{
	Sleep(sec*1000);
}//ThreadSleep


#endif
