/***************************************************************************
                          X3270Connection.h  -  description
                             -------------------
    begin                : Tue May 14 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _X3270CONNECTION_H_
#define _X3270CONNECTION_H_
#ifdef USE_X3270

#include "ConnectionInterface.h"
#include "../MiniXml/MiniXML.h"


class X3270Connection :  public ConnectionInterface
{
	private:
		int m_connectScriptDone;
		int m_haveEmulatorOpen;
		int m_closeTerminalWindow;
		int m_connected;
		int m_scriptfileParsed;
		int m_timeOutSec;
		char m_symbolicConnName[65];
		char m_hostName[_MAX_PATH];
		MiniXML m_miniXML;
		FILE *m_x3270fp;
		int m_x3270resp;
		char m_x3270buffer[3841];

		int parseCommands(Element *, Element *);
		int parse(char *);
		int firstSteps();
		int lastSteps();
		int x3270Wait(int);
		int getX3270Response();
		int sendString(char *);
		int copyPs2Str(char *, long);
		int copyStr2Ps(char *, long);
		int copyStr2Ps(char *, long, long);
		int setCursor(long);
		int sendClearKey();
		int sendResetKey();
		int sendEnterKey();
		int sendPaKey(int);
		int sendTabKey();
		int sendBacktabKey();
		int sendPfKey(int);
		int sendHomeKey();
		int sendCursorLeftKey();
		int sendCursorRightKey();
		int sendCursorUpKey();
		int sendCursorDownKey();

	public:
		X3270Connection();
		~X3270Connection();

		int openConnection(ConnectionId *);
		int closeConnection(ConnectionId *);
		int setParameter(ConnParameter *);
		int getParameter(ConnParameter *);
		ConnParameter **getAllOutputParameters(char *);
		int doScriptCommand(char *);
		void parseScriptFile(char *);
		int isConnected() {return m_connected;};

		int openTerminalWindow(ConnectionId *);
		int closeTerminalWindow(ConnectionId *);
		void setDisconnectCloseTerminalWindow() {m_closeTerminalWindow = 1;}
};//X3270Connection


#define X3270_RC_SUCCESS 0        // Good return code.
#define X3270_RC_INVALID_FP 1     // Invalid file pointer to x3270 emulator.
#define X3770_RC_ERR_RESPONSE 2   // Error in response from the x3270 emulator.
#define X3270_RC_BAD_PARAM 3      // Bad parameter.
#define X3270_RC_SYSTEM_ERROR 4   // System error.
#define X3270_RC_INVALID_PS 5     // Invalid PS.
#define X3270_RC_TERM_OPEN_ERR 6  // Open terminal window error.
#define X3270_RC_TERM_NOT_CONN 7  // Terminal is not connected.


#endif
#endif
